//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Program        : Collatz
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  University of California, Irvine
//                  Irvine, CA 92617-3435
//                  e-mail: pattis@ics.uci.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   The Collatz conjecture states: Any positive number will be eventually
// be reduced to 1 by repeating the following process
//
//   o If a number is even, divide it by 2
//   o If a number is odd,  multiply it by 3 and add 1
//     (the number will now be even, so it will next be divided by 2)
//
//   This program prompts the user for a legal starting value and implements
// the Collatz process, displaying the sequence of values guenerated by the
// process. When the number is reduced to 1, the program reports the total
// number of cycles it took (how many times it applied the process).
//
// Known Bugs     : If during the process, the number becomes too big, it
//                    overflows to a negative value; currently the program
//                    detects this problem and then terminates, without
//                    completing the process.
//                  This problem is a result of Java int type.
//
// Future Plans   : Ensure user enters a positive value initially
//                  Use Infinite Precision integers so no overflow
//
// Program History:
//   5/18/01: R. Pattis - Operational for 15-100
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////



import edu.uci.ics.pattis.introlib.Prompt;


public class Application {


	public static void main(String[] args) {
		int cycleCount     = 1;
		int originalNumber = Prompt.forInt("Enter a positive integer");
		int testNumber     = originalNumber;


		//Implement Collatz process (as described above)

		for(;;) {
			//Display current status
			System.out.println("Cycle " + cycleCount + ": testNumber is now " + testNumber);

			////////////////////
			if (testNumber == 1)              //Check for termination
				break;
			////////////////////

			//Update status
			cycleCount++;
			if (testNumber%2 == 0)            //Is even if remainder after /2 is 0
				testNumber = testNumber/2;
			else {  
				testNumber = 3*testNumber + 1;
				//Ensure increase doesn't overflow the "int" type (in which case testNumber
				//  magically becomes negative)
				if (testNumber <= 0)
					throw new IllegalStateException("\nArithmetic Overflow to " + testNumber);
			}
		}


		//Display statistics gathered during Collatz process

		System.out.println("\n\nFor Test Number = " + originalNumber + ", Cycles to 1 = " + cycleCount + "\n");
	}
}
